-- SprayScan
-- Spec for scanning fertilizer rate on fields and setting sprayers usage
-- V1.0
-- 12.12.13


SprayScan = {};

function SprayScan.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Sprayer, specializations);
end;

function SprayScan:load(xmlFile)
	
	self.getCuttingAreas = SpecializationUtil.callSpecializationsFunction("self.getCuttingAreas");
	self.wwMinMaxAI = SpecializationUtil.callSpecializationsFunction("self.wwMinMaxAI");
	self.wwMinMaxAreas = SpecializationUtil.callSpecializationsFunction("self.wwMinMaxAreas");
	self.createScanArea = SpecializationUtil.callSpecializationsFunction("self.createScanArea");
	self.setScanArea = SpecializationUtil.callSpecializationsFunction("self.setScanArea");
	
	self.getCuttingAreas = SprayScan.getCuttingAreas;
	self.wwMinMaxAI = SprayScan.wwMinMaxAI;
	self.wwMinMaxAreas = SprayScan.wwMinMaxAreas;
	self.createScanArea = SprayScan.createScanArea;
	self.setScanArea = SprayScan.setScanArea;
	
	self.origSprayLitersPerSecond = table.shallow_copy(self.sprayLitersPerSecond);
	
	-- Areas
	
	self.wwMin = 0;
	self.wwMax = 0;
	self.wwY = 0;
	self.wwZ = 0;
	self.scanOffsetZ = 2.5;
	
	self.ScanArea = {}
	self.ScanArea = self:createScanArea();
	
end;

function table.shallow_copy(t)
  local t2 = {}
  for k,v in pairs(t) do
    t2[k] = v
  end
  return t2
end

function SprayScan:delete()
end;

function SprayScan:readStream(streamId, connection)

end;

function SprayScan:writeStream(streamId, connection)

end;

function SprayScan:mouseEvent(posX, posY, isDown, isUp, button)

end;

function SprayScan:keyEvent(unicode, sym, modifier, isDown)
end;

function SprayScan:onLeave()

end

function SprayScan:update(dt)
    if self.isClient then
        if self.allowsSpraying and self:getIsActiveForInput() then
            if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA) then
                self:getCuttingAreas();
				self:setScanArea();
            end;
        end;
    end;
end;

function SprayScan:updateTick(dt)
	if self.isTurnedOn and self.allowsSpraying then
		if self.isServer then
			
			for _,area in pairs(self.ScanArea) do
				local x,y,z = getWorldTranslation(area.start);
				local x1,y1,z1 = getWorldTranslation(area.width);
				local x2,y2,z2 = getWorldTranslation(area.height);
				
				local sprayedArea = Utils.getDensity(g_currentMission.terrainDetailId, g_currentMission.sprayChannel, x, z, x1, z1, x2, z2);
				
				if sprayedArea > 5 then
					if self.currentFillType == (Fillable.FILLTYPE_LIQUIDMANURE or Fillable.FILLTYPE_MANURE) then
						self.sprayLitersPerSecond[self.currentFillType] = self.origSprayLitersPerSecond[self.currentFillType]-sprayedArea/5;
					else
						self.sprayLitersPerSecond[self.currentFillType] = self.origSprayLitersPerSecond[self.currentFillType]-sprayedArea/100;
					end;
				else
					self.sprayLitersPerSecond[self.currentFillType] = self.origSprayLitersPerSecond[self.currentFillType];
				end;
			end;
		end;
	end;
end;


function SprayScan:draw()

end;

function SprayScan:getCuttingAreas()
	self.wwMin = 0;
	self.wwMax = 0;
	self.wwY = 0;
	self.wwZ = 0;
	self.wwMin,self.wwMax,self.wwY,self.wwZ = self:wwMinMaxAreas(self,self.cuttingAreas);
	for nIndex,oImplement in pairs(self.attachedImplements) do --parse all implements
		if oImplement ~= nil and oImplement.object ~= nil then
			self.wwMin,self.wwMax,self.wwY,self.wwZ = self:wwMinMaxAreas(self,oImplement.object.cuttingAreas);
		end;
	end;
	
	local workWidth = math.abs(self.wwMax-self.wwMin);
	if workWidth > .1 then
		self.workWidth = workWidth;
		self.wwCenter = (self.wwMin+self.wwMax)/2;
		if math.abs(self.wwCenter) < 0.1 then
			self.wwCenter = 0;
		end;
	end;
end;

function SprayScan:wwMinMaxAreas(self,areas)

	if areas ~= nil then
		for _,cuttingArea in pairs(areas) do
			if self:getIsAreaActive(cuttingArea) then

				local x1,y1,z1 = getWorldTranslation(cuttingArea.start)
				local x2,y2,z2 = getWorldTranslation(cuttingArea.width)
				local x3,y3,z3 = getWorldTranslation(cuttingArea.height)
				local lx1,ly1,lz1 = worldToLocal(self.rootNode,x1,y1,z1)
				local lx2,ly2,lz2 = worldToLocal(self.rootNode,x2,y2,z2)
				local lx3,ly3,lz3 = worldToLocal(self.rootNode,x3,y3,z3)
				local caParentId = getParent(cuttingArea.start)
				
				if lx1 < self.wwMin then
					self.wwMin = lx1;
				end
				if lx1 > self.wwMax then
					self.wwMax = lx1;
				end
				if lx2 < self.wwMin then
					self.wwMin = lx2;
				end
				if lx2 > self.wwMax then
					self.wwMax = lx2;
				end
				if lx3 < self.wwMin then
					self.wwMin = lx3;
				end
				if lx3 > self.wwMax then
					self.wwMax = lx3;
				end
				self.wwY = ly1;
				self.wwZ = lz1;
				self.caParentId = caParentId;
			end
		end
	end
	return self.wwMin, self.wwMax, self.wwY, self.wwZ;

end;

function SprayScan:createScanArea()
	local ScanArea = {};
	local startId = createTransformGroup("start");
	link(self.components[1].node, startId);
	local heightId = createTransformGroup("height");
	link(self.components[1].node, heightId);
	local widthId = createTransformGroup("width");
	link(self.components[1].node, widthId);

	table.insert(ScanArea, {foldMinLimit=0,start=startId,height=heightId,foldMaxLimit=0.2,width=widthId});

	return ScanArea;
end;

function SprayScan:setScanArea()
	local xMin = self.wwMin;
	local xMax = self.wwMax;
	local y = self.wwY;
	local z = self.wwZ + self.scanOffsetZ;
	local hz = z - 2;

	setTranslation(self.ScanArea[1].start,xMax,y,z);
	setTranslation(self.ScanArea[1].height,xMax,y,hz);
	setTranslation(self.ScanArea[1].width,xMin,y,z);
end;